package com.myco.jmstest;

import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicLong;

import org.apache.log4j.Logger;
import org.springframework.jms.JmsException;

public abstract class AbstractJmsSender {

    private static Logger LOGGER = Logger.getLogger(AbstractJmsSender.class);

    private long lastPrintedTime = 0;
    
    private AtomicBoolean currentlyPrinting = new AtomicBoolean(false);
    private AtomicLong messagesSent = new AtomicLong();
    private AtomicLong totalSent = new AtomicLong();
    private boolean isShutdown = false;
    
    private final LinkedBlockingQueue<String> sendQueue = new LinkedBlockingQueue<String>(10000);

    public AbstractJmsSender(int numThreads) {
        for(int i = 0; i < numThreads; i++) {
            Thread thread = new Thread(new JmsSenderWorker(), "JmsSender-" + i);
            thread.setDaemon(true);
            thread.start();
        }
    }
    
    protected abstract void send(String msg);
    
    private class JmsSenderWorker implements Runnable {
        @Override
        public void run() {
            while(true) {
                
                if(isShutdown) {
                    break;
                }
                
                try {
                    String msg = sendQueue.take();
                    if(msg != null) {
                        try {
                            
                            send(msg);
                            
                            messagesSent.incrementAndGet();
                            totalSent.incrementAndGet();
                            
                            long currentTime = System.currentTimeMillis();
                            if(currentTime - lastPrintedTime > 5000) {
                                if(currentlyPrinting.compareAndSet(false, true)) {
                                    if(currentTime - lastPrintedTime > 5000) {
                                        long numSent = messagesSent.getAndSet(0);
                                        LOGGER.info("Messages sent: " + numSent);
                                        lastPrintedTime = currentTime;
                                    }
                                    currentlyPrinting.set(false);
                                }
                            }
                        } catch (JmsException e) {
                            e.printStackTrace();
                        }
                    }
                } catch(InterruptedException e) {
                    Thread.currentThread().interrupt();
                    break;
                }
            }
        }
    }
    
    public final void sendMessage(String msg) throws InterruptedException {
        while(true) {
            if(sendQueue.offer(msg)) {
                break;
            }
            Thread.sleep(1);
        }
    }
    
    public long getTotalSent() {
        return totalSent.get();
    }
    
    public void shutdown() throws InterruptedException {
        isShutdown = true;
    }
}
